/***************************************************************************
 *   Copyright (C) 2003 by Stefano Rivoir                                  *
 *   s.rivoir@gts.it                                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <kmainwindow.h>
#include <klocale.h>
#include <kapplication.h>
#include <kstdaccel.h>
#include <kaction.h>
#include <kactionclasses.h>
#include <kaboutapplication.h>
#include <kmessagebox.h>
#include <dcopclient.h>
#include <qcursor.h>
#include <qtooltip.h>

#include "kpreset.h"
#include "kmixer.h"
#include "kamix.h"
#include "kconfigwin.h"
#include "kmixeritem.h"
#include "kmasterview.h"
#include "kvumeter.h"
#include "kosd.h"
#include "kamixconfig.h"
#include "kalsa.h"

#include "config.h"

#ifdef VUMETER
#include <arts/kartsdispatcher.h>
#include <arts/kartsserver.h>
#include <arts/artsflow.h>
#endif

kamix::kamix()
    : DCOPObject ( "KamixDCOPIface" ),
    KMainWindow( 0, "kamix" )
      
{

    // set the shell's ui resource file
    setXMLFile("kamixui.rc");
    
    KamixConfig::self()->readConfig();

    _doclose = false;

    QHBox *hb = new QHBox ( this );
    setCentralWidget(hb);

#ifdef VUMETER
    vu = new KVUMeter ( hb );
#endif

    vlayout = new QVBox ( hb );
    
    initAlsa();

    KShortcut sh1, sh2;
    QString s1, s2;
    
    sh1.init( Qt::Key_PageUp );
    sh2.init( Qt::Key_PageDown );
    
    s1 = KamixConfig::self()->volUpShortcut;
    s2 = KamixConfig::self()->volDnShortcut;
    if ( s1!="" )
        sh1.init ( s1 );
    if ( s2!="" )
        sh2.init ( s2 );
    
    KAction *a1 = new KAction ( i18n("Quit"), "exit", 
        KStdAccel::shortcut(KStdAccel::Quit),
    	this, SLOT(quit()), actionCollection(), "exit" );
    KAction *a2 = new KAction ( i18n("Options"), "configure",
    	KStdAccel::shortcut(KStdAccel::AccelNone),
        this, SLOT(showConfig()), actionCollection(), "options" );
    KAction *aUp = new KAction ( i18n("Volume up"), "up", 
        sh1,
        this, SLOT(volUp()), actionCollection(), "up" );
    KAction *aDown = new KAction ( i18n("Volume down"), "down", 
        sh2,
        this, SLOT(volDown()), actionCollection(), "down" );
    KAction *aAbout = new KAction ( i18n("About kamix"), "help",
        KStdAccel::shortcut(KStdAccel::AccelNone),
        this, SLOT(showAbout()), actionCollection(), "help" );
	
#ifdef VUMETER
    KAction *aToggleScope = new KAction ( i18n("Show level meter"), "artsfftscope",
        KStdAccel::shortcut(KStdAccel::AccelNone),
        this, SLOT(toggleMeter()), actionCollection(), "scope" );
    if ( !KamixConfig::self()->useVumeter ) 
        aToggleScope->setEnabled( false );
#endif

    KAction *aToggleViewV = new KAction ( i18n("Toggle view"), "view_top_bottom",
        KStdAccel::shortcut(KStdAccel::AccelNone),
        this, SLOT(toggleViewTypeV()), actionCollection(), "toggleviewv" );
    KAction *aToggleViewH = new KAction ( i18n("Toggle view"), "view_left_right",
        KStdAccel::shortcut(KStdAccel::AccelNone),
        this, SLOT(toggleViewTypeH()), actionCollection(), "toggleviewh" );

    aPst = new KToolBarPopupAction ( i18n("Presets"), "down" );
    aPst->setIcon ( "project_open" );
    aPst->setDelayed ( false );
    connect ( aPst->popupMenu(), SIGNAL(activated(int)), this, SLOT(presetsPopup(int)));
    
    aUp->plug ( toolBar() );
    aDown->plug ( toolBar() );
    
    toolBar()->insertLineSeparator();
    
#ifdef VUMETER
    aToggleScope->plug ( toolBar() );
#endif

    toolBar()->insertLineSeparator();
    a2->plug ( toolBar() );
    aPst->plug ( toolBar() );
    aToggleViewV->plug ( toolBar() );
    aToggleViewH->plug ( toolBar() );
    
    toolBar()->insertLineSeparator();
    aAbout->plug ( toolBar() );
    a1->plug ( toolBar() );
    
    //aTest->plug ( toolBar() );
    
    timer = new QTimer(this);
    timer2= new QTimer(this);
    
#ifdef VUMETER
    if ( KamixConfig::self()->useVumeter ) {
        adpt = new KArtsDispatcher();
        asvr = new KArtsServer();

        canShowMeter = true;
        if ( asvr->server().isNull() ) {
            aToggleScope->setEnabled(false);
	    showMeter = false;
	    canShowMeter = false;
        } else
            vc = asvr->server().outVolume();
    }
#else
    canShowMeter = false;
#endif

    timer->changeInterval ( 250 );
    timer2->changeInterval ( 20 );

    readConfig();

    if ( KamixConfig::self()->restoreWindow ) {
        setGeometry ( KamixConfig::self()->windowGeometry );
        move ( KamixConfig::self()->windowPosition );
    }

    initControls();

    tray = new KamixSystemTray ( this );
    tray->setPixmap ( tray->loadIcon ("kamix.png" ));

    connect ( tray, SIGNAL(quitSelected()), this, SLOT(quit()) );   
    connect ( tray, SIGNAL(midButtonPressed()), this, SLOT(midButtonPressed()) );
    connect ( tray, SIGNAL(wheelUp()), this, SLOT(volUp() ));
    connect ( tray, SIGNAL(wheelDown()), this, SLOT(volDown() ));
    tray->show();

    connect ( timer, SIGNAL (timeout()), this, SLOT(readMixer()) );
    connect ( timer2,SIGNAL (timeout()), this, SLOT(readLevel()) );
    updateTrayTip();

}

kamix::~kamix()
{
}

/*
void kamix::dotest() {
    osd.setText( "Stefano" );
    osd.showOsd();
}
*/

int kamix::initAlsa() {
    int res;

    res = KAlsa::self()->init();
    if ( res )
        return res;
    alsa_callback_setup ( this );
    return 0;
}

void kamix::toggleViewType ( KMixerView::viewType t ) {
    unsigned int i;
    KMixerView::viewType t2;
    
    t2 = views.at(0)->type;
    if ( t != t2 )
        t2 = t;
    else
        t2 = KMixerView::viewTabs;

    for ( i=0; i<views.count(); i++ ) {
        views.at(i)->type = t2;
        views.at(i)->initControls();
    }
}

void kamix::toggleViewTypeV() {
    toggleViewType ( KMixerView::viewTiledVert );
}

void kamix::toggleViewTypeH() {
    toggleViewType ( KMixerView::viewTiledHor );
}

void kamix::toggleMeter() {
#ifdef VUMETER
    if ( !showMeter ) {
        showMeter = true;
	timer2->start( 20 );
	vu->show();
    } else {
        showMeter = false;
	timer2->stop();
	vu->hide();
    }
#endif
}

void kamix::readConfig ()
{
    unsigned int i;
    
    if ( KPreset::count()>0 ) {
        aPst->setEnabled(true);
	aPst->popupMenu()->clear();
	for ( i=0; i<KPreset::count(); i++ ) {
	    aPst->popupMenu()->insertItem(KPreset::preset(i));
        }
    } else
	aPst->setEnabled(false);

}

void kamix::reinitApp() {
    KamixConfig::self()->allowLoadLevels = false;
    KAlsa::self()->init();    
    readConfig();
    initControls();
    setUpdatesEnabled( true );
    repaint();
    KamixConfig::self()->allowLoadLevels = true;
}

void kamix::initControls ()
{
    
#ifdef VUMETER
    showMeter = KamixConfig::self()->showMeter;
    
    if ( !canShowMeter || !showMeter || !KamixConfig::self()->useVumeter ) {
	timer2->stop();
        vu->hide();
    }
#else
    showMeter = false;
#endif

    unsigned int i;
    for ( i=0; i<views.count(); i++ )
        delete views.at(i);

    KamixConfig::self()->config()->sync();
    
    views.clear();
    
    KMixerView *v;
    
    for ( i=0; i<KAlsa::self()->cardsCount(); i++ ) {
    
    	if ( !KamixConfig::self()->cardHidden( KAlsa::self()->card(i)->configSection() )) {
 	
	    switch ( KamixConfig::self()->viewType ) {
	    case 1:
                v = new KMixerView(KAlsa::self()->card(i)->mixer(), KMixerView::viewTiledVert, vlayout, "", this );
	        break;
	    case 2:
                v = new KMixerView(KAlsa::self()->card(i)->mixer(), KMixerView::viewTiledHor, vlayout, "", this );
	        break;
	    default:
	        v = new KMixerView(KAlsa::self()->card(i)->mixer(), KMixerView::viewTabs, vlayout, "", this );
	        break;
	    }
	    views.append(v);
	    v->initControls();
	    v->show();
	}
    }
    
}

KMixer* kamix::activeMixer() {
    return KAlsa::self()->card(activeCard)->mixer();
}

void kamix::showConfig()
{
    KConfigWin *cw = new KConfigWin( this );
    if ( cw->exec() )
        reinitControls();
}

void kamix::reinitControls()
{
    KamixConfig::self()->writeConfig();
    readConfig();
    initControls();
}

void kamix::readMixer()
{
    static int icon = -1;
    
    // Processes external ALSA events
    unsigned int i;
    for ( i=0; i<KAlsa::self()->cardsCount(); i++ )
        snd_mixer_handle_events(KAlsa::self()->card(i)->mixer()->amix);

    KMixerItem *itm = KAlsa::self()->shortItem(KAlsa::self()->defaultCard());
    
    if ( itm && itm->muted(KMixConst::Playback, KAMIX_ALL_CHANNELS)==KMixConst::Muted && icon != 0 ) {
        tray->setPixmap ( tray->loadIcon ("kamix_mute.png" ));
	icon = 0;
    }
    if ( itm && itm->muted(KMixConst::Playback, KAMIX_ALL_CHANNELS)!=KMixConst::Muted && icon != 1 ) {
        tray->setPixmap ( tray->loadIcon ("kamix.png" ));
	icon = 1;
    }
}

void kamix::updateTrayTip() {
    KMixerItem *itm = KAlsa::self()->shortItem(KAlsa::self()->defaultCard());
    long t = KamixConfig::self()->volumeMode;
    
    if ( itm ) {
        QString s2;
        QString s = itm->name();
	s += " "+i18n("at")+" ";
	if ( t == 0 )
		s += s2.sprintf ( "%ld", itm->volume(KMixConst::Playback) );
	if ( t == 1 )
		s += s2.sprintf ( "%ld%%", itm->percent(KMixConst::Playback) );
#ifdef HAS_SND_DECIBEL
	if ( t == 2 )
		s += s2.sprintf ( "%ld dB", itm->dB(KMixConst::Playback) );
#endif
        switch ( itm->muted(KMixConst::Playback, KAMIX_ALL_CHANNELS )) {
        case KMixConst::Muted:
	    s += i18n(", muted" );
	    break;
	case KMixConst::Partial:
	    s += i18n(", partially muted" );
	}
        QToolTip::add ( tray, s );
    }
}

void kamix::readLevel()
{
#ifdef VUMETER
    if ( KamixConfig::self()->useVumeter ) {
        vu->setValueLeft ( vc.currentVolumeLeft() );
        vu->setValueRight( vc.currentVolumeRight() );
    }
#endif
}

void kamix::quit()
{
    _doclose = true;
    timer->stop();
    close();
    exit(0);
}

void kamix::volDown() {
    KMixerItem *itm = KAlsa::self()->shortItem(KAlsa::self()->defaultCard());
    if ( itm ) {
        long l = itm->volume(KMixConst::Playback);
        itm->setVolume( KMixConst::Playback, --l );
    }
}

void kamix::volUp() {
    KMixerItem *itm = KAlsa::self()->shortItem(KAlsa::self()->defaultCard());
    if ( itm ) {
        long l = itm->volume(KMixConst::Playback);
        itm->setVolume( KMixConst::Playback, ++l );
    }
}

bool kamix::queryExit() {
    // Closes objects and save settings
    
    QString s;
    unsigned int i;
    for ( i=0; i<KAlsa::self()->cardsCount(); i++ )
        KAlsa::self()->card(i)->writeConfig();
    
    return true;
}

void kamix::showAbout() {
    KAboutApplication *dlg = new KAboutApplication(this);
    dlg->show(this);
}

void kamix::closeEvent ( QCloseEvent *ev ) {
   
    if ( !_doclose && !ev->spontaneous() ) {

	if ( !KamixConfig::self()->trayWarning ) {
           KMessageBox::information ( this,
	   	i18n("Closing this window will NOT quit the application, it will be put "
		"the systray instead. If you really want to quit, press the Quit "
		"button on the toolbar, or select Quit on the menu of the systray"),
		i18n("Warning!"),
		"TrayWarning");
        }
       
        ev->ignore();
        this->hide();
    } else {
    
        // Forces views delete to store splitted values
        unsigned int i;
        for ( i=0; i<views.count(); i++ )
            delete views.at(i);
	
        KamixConfig::self()->showMeter = showMeter;
        KamixConfig::self()->windowGeometry = geometry();
        KamixConfig::self()->windowPosition = pos();
	KamixConfig::self()->restoreToSystray = isHidden();
        KamixConfig::self()->writeConfig();
        KamixConfig::self()->config()->sync();
	
        ev->accept();
    }
   
}

void kamix::showMaster() {
    KMixerItem *itm = KAlsa::self()->shortItem(KAlsa::self()->defaultCard());
    if ( itm ) {
        osdDisable();
    	KMasterView *v = new KMasterView ( itm, this );
    	v->popup ( QCursor::pos() );
    }

}

void kamix::midButtonPressed() {
    if ( KamixConfig::self()->midButtonMutes )
        masterMute();
    else
        showMaster();
}

void kamix::presetsPopup(int i) {
    KPreset::loadPreset(aPst->popupMenu()->text(i));
}

void kamix::osdEnable() {
    unsigned int i;
    for ( i=0; i<views.count(); i++ )
        views.at(i)->osdEnabled = true;
}
void kamix::osdDisable() {
    unsigned int i;
    for ( i=0; i<views.count(); i++ )
        views.at(i)->osdEnabled = false;
}

// DCOP INTERFACE FUNCTIONS
void kamix::masterVolDown() {
    volDown();
}

void kamix::masterVolUp() {
    volUp();
}

void kamix::masterMute() {
    KMixerItem *itm = KAlsa::self()->shortItem(KAlsa::self()->defaultCard());
    if ( itm )
        itm->toggleMuted( KMixConst::Playback );
}

void kamix::masterShow() {
    showMaster();
}

void kamix::quitKamix() {
    quit();
}

#include "kamix.moc"
